@@ -15,6 +15,11 @@
 #include "SoapyRemoteDefs.hpp"
 #include "SoapyHTTPUtils.hpp"
 #include "SoapyRPCSocket.hpp"
+#include "SoapyIfAddrs.hpp"
+#include <csignal> //sig_atomic_t
+#include <memory> //unique_ptr
+#include <vector>
+#include <mutex>
 #include <thread>
 #include <ctime>
 #include <chrono>
@@ -46,17 +51,18 @@
 //! Service stopped, use with multicast NOTIFY
 #define NTS_BYEBYE "ssdp:byebye"
 
-struct SoapySSDPEndpointData
+/***********************************************************************
+ * Utility functions
+ **********************************************************************/
+static std::string uuidFromUSN(const std::string &usn)
 {
-    int ipVer;
-    SoapyRPCSocket sock;
-    std::string groupURL;
-    std::thread *thread;
-    std::chrono::high_resolution_clock::time_point lastTimeSearch;
-    std::chrono::high_resolution_clock::time_point lastTimeNotify;
-    typedef std::map<std::string, std::pair<std::string, std::chrono::high_resolution_clock::time_point>> DiscoveredURLs;
-    DiscoveredURLs usnToURL;
-};
+    auto posUUID = usn.find("uuid:");
+    if (posUUID == std::string::npos) return usn;
+    posUUID += 5;
+    const auto posColon = usn.find(":", posUUID);
+    if (posColon == std::string::npos) return usn;
+    return usn.substr(posUUID, posColon-posUUID);
+}
 
 static std::string timeNowGMT(void)
 {
@@ -66,90 +72,57 @@ static std::string timeNowGMT(void)
     return std::string(buff, len);
 }
 
-std::shared_ptr<SoapySSDPEndpoint> SoapySSDPEndpoint::getInstance(void)
+/***********************************************************************
+ * Storage for SSDP endpoint
+ **********************************************************************/
+struct SoapySSDPEndpointImpl
 {
-    static std::mutex singletonMutex;
-    std::lock_guard<std::mutex> lock(singletonMutex);
-    static std::weak_ptr<SoapySSDPEndpoint> epWeak;
-    auto epShared = epWeak.lock();
-    if (not epShared) epShared.reset(new SoapySSDPEndpoint());
-    epWeak = epShared;
-    return epShared;
-}
-
-SoapySSDPEndpoint::SoapySSDPEndpoint(void):
-    serviceRegistered(false),
-    periodicSearchEnabled(false),
-    periodicNotifyEnabled(false),
-    done(false)
-{
-    const bool isIPv6Supported = not SoapyRPCSocket(SoapyURL("tcp", "::", "0").toString()).null();
-    this->spawnHandler("0.0.0.0", SSDP_MULTICAST_ADDR_IPV4, 4);
-    if (isIPv6Supported) this->spawnHandler("::", SSDP_MULTICAST_ADDR_IPV6, 6);
-}
-
-SoapySSDPEndpoint::~SoapySSDPEndpoint(void)
-{
-    done = true;
-    for (auto &data : handlers)
+    SoapySSDPEndpointImpl(void):
+        thread(nullptr),
+        done(false)
     {
-        data->thread->join();
-        delete data->thread;
-        delete data;
-    }
-}
-
-void SoapySSDPEndpoint::registerService(const std::string &uuid, const std::string &service)
-{
-    std::lock_guard<std::mutex> lock(mutex);
-    this->serviceRegistered = true;
-    this->uuid = uuid;
-    this->service = service;
-}
-
-void SoapySSDPEndpoint::enablePeriodicSearch(const bool enable)
-{
-    std::lock_guard<std::mutex> lock(mutex);
-    periodicSearchEnabled = enable;
-    if (not enable) return; //quiet on disable
-    for (auto &data : handlers) this->sendSearchHeader(data);
-}
-
-void SoapySSDPEndpoint::enablePeriodicNotify(const bool enable)
-{
-    std::lock_guard<std::mutex> lock(mutex);
-    periodicNotifyEnabled = enable;
-    if (not enable) return; //quiet on disable
-    for (auto &data : handlers) this->sendNotifyHeader(data, NTS_ALIVE);
-}
-
-std::vector<std::string> SoapySSDPEndpoint::getServerURLs(const int ipVer, const bool only)
-{
-    std::lock_guard<std::mutex> lock(mutex);
-
-    //create a single mapping of discovered URLs using the preferences specified
-    SoapySSDPEndpointData::DiscoveredURLs usnPrefToURL;
-    for (auto &data : handlers)
-    {
-        const bool ipVerMatch = data->ipVer == ipVer;
-        //ignore this data set if only is specified and the IP version does not match
-        if (only and not ipVerMatch) continue;
-        for (auto &pair : data->usnToURL)
-        {
-            //ignore this URL if the entry is already present and the IP version does not match
-            if (usnPrefToURL.count(pair.first) != 0 and not ipVerMatch) continue;
-            usnPrefToURL[pair.first] = pair.second;
-        }
+        return;
     }
 
-    //copy the filtered URLs into the resulting list
-    std::vector<std::string> serverURLs;
-    for (auto &pair : usnPrefToURL) serverURLs.push_back(pair.second.first);
-    return serverURLs;
-}
+    SoapySocketSession sess;
 
-void SoapySSDPEndpoint::spawnHandler(const std::string &bindAddr, const std::string &groupAddr, const int ipVer)
+    //handler thread
+    std::thread *thread;
+
+    //protection between threads
+    std::mutex mutex;
+
+    //server data
+    std::vector<SoapySSDPEndpointData *> handlers;
+
+    //signal done to the thread
+    sig_atomic_t done;
+
+    //active USNs per IP version
+    typedef std::map<std::string, std::pair<std::string, std::chrono::high_resolution_clock::time_point>> DiscoveredURLs;
+    std::map<int, DiscoveredURLs> usnToURL;
+};
+
+/***********************************************************************
+ * Storage per network interface
+ **********************************************************************/
+struct SoapySSDPEndpointData
 {
+    int ipVer;
+    SoapyRPCSocket sock;
+    std::string groupURL;
+    std::string ethAddr;
+    std::string ethName;
+    std::chrono::high_resolution_clock::time_point lastTimeSearch;
+    std::chrono::high_resolution_clock::time_point lastTimeNotify;
+
+    static SoapySSDPEndpointData *setupSocket(const std::string &bindAddr, const std::string &groupAddr, const SoapyIfAddr &ifAddr);
+};
+
+SoapySSDPEndpointData *SoapySSDPEndpointData::setupSocket(const std::string &bindAddr, const std::string &groupAddr, const SoapyIfAddr &ifAddr)
+{
+    std::unique_ptr<SoapySSDPEndpointData> data(new SoapySSDPEndpointData());
+
     //static list of blacklisted groups
     //if we fail to join a group, its blacklisted
     //so future instances wont get the same error
@@ -157,24 +130,23 @@ void SoapySSDPEndpoint::spawnHandler(const std::string &bindAddr, const std::str
     static std::set<std::string> blacklistedGroups;
 
     //check the blacklist
-    if (blacklistedGroups.find(groupAddr) != blacklistedGroups.end())
+    if (blacklistedGroups.find(ifAddr.addr) != blacklistedGroups.end())
     {
-        SoapySDR::logf(SOAPY_SDR_DEBUG, "SoapySSDPEndpoint::spawnHandler(%s) group blacklisted due to previous error", groupAddr.c_str());
-        return;
+        SoapySDR::logf(SOAPY_SDR_DEBUG, "SoapySSDPEndpoint::setupSocket(%s) interface blacklisted due to previous error", ifAddr.addr.c_str());
+        return nullptr;
     }
 
-    auto data = new SoapySSDPEndpointData();
-    data->ipVer = ipVer;
+    SoapySDR::logf(SOAPY_SDR_DEBUG, "SoapySSDP join multicast endpoint on %s IPv%d %s", ifAddr.name.c_str(), ifAddr.ipVer, ifAddr.addr.c_str());
+    data->ipVer = ifAddr.ipVer;
     auto &sock = data->sock;
 
     const auto groupURL = SoapyURL("udp", groupAddr, SSDP_UDP_PORT_NUMBER).toString();
-    int ret = sock.multicastJoin(groupURL);
+    int ret = sock.multicastJoin(groupURL, ifAddr.addr, {ifAddr.addr});
     if (ret != 0)
     {
-        blacklistedGroups.insert(groupAddr);
-        SoapySDR::logf(SOAPY_SDR_WARNING, "SoapySSDPEndpoint failed join group %s\n  %s", groupURL.c_str(), sock.lastErrorMsg());
-        delete data;
-        return;
+        blacklistedGroups.insert(ifAddr.addr);
+        SoapySDR::logf(SOAPY_SDR_WARNING, "SoapySSDPEndpoint failed join group %s on %s\n  %s", groupURL.c_str(), ifAddr.name.c_str(), sock.lastErrorMsg());
+        return nullptr;
     }
 
     const auto bindURL = SoapyURL("udp", bindAddr, SSDP_UDP_PORT_NUMBER).toString();
@@ -182,28 +154,123 @@ void SoapySSDPEndpoint::spawnHandler(const std::string &bindAddr, const std::str
     if (ret != 0)
     {
         SoapySDR::logf(SOAPY_SDR_ERROR, "SoapySSDPEndpoint::bind(%s) failed\n  %s", bindURL.c_str(), sock.lastErrorMsg());
-        delete data;
-        return;
+        return nullptr;
     }
 
     data->groupURL = groupURL;
-    data->thread = new std::thread(&SoapySSDPEndpoint::handlerLoop, this, data);
-    handlers.push_back(data);
+    data->ethAddr = ifAddr.addr;
+    data->ethName = ifAddr.name;
+    return data.release();
 }
 
-void SoapySSDPEndpoint::handlerLoop(SoapySSDPEndpointData *data)
+/***********************************************************************
+ * SoapySSDPEndpoint implementation
+ **********************************************************************/
+SoapySSDPEndpoint::SoapySSDPEndpoint(void):
+    _impl(new SoapySSDPEndpointImpl()),
+    serviceIpVer(SOAPY_REMOTE_IPVER_NONE),
+    periodicSearchEnabled(false),
+    periodicNotifyEnabled(false)
 {
-    auto &sock = data->sock;
+    const bool isIPv6Supported = not SoapyRPCSocket(SoapyURL("tcp", "::", "0").toString()).null();
+    for (const auto &ifAddr : listSoapyIfAddrs())
+    {
+        SoapySDR::logf(SOAPY_SDR_TRACE, "Interface %d: %s [addr=%s, up?%d, loop?%d, mcast?%d]", ifAddr.ethno, ifAddr.name.c_str(), ifAddr.addr.c_str(), ifAddr.isUp, ifAddr.isLoopback, ifAddr.isMulticast);
+        if (not ifAddr.isUp) continue;
+        if (ifAddr.isLoopback) continue;
+        if (not ifAddr.isMulticast) continue;
+        SoapySSDPEndpointData *data(nullptr);
+        if (ifAddr.ipVer == 4)                     data = SoapySSDPEndpointData::setupSocket("0.0.0.0", SSDP_MULTICAST_ADDR_IPV4, ifAddr);
+        if (ifAddr.ipVer == 6 and isIPv6Supported) data = SoapySSDPEndpointData::setupSocket("::",      SSDP_MULTICAST_ADDR_IPV6, ifAddr);
+        if (data != nullptr) _impl->handlers.push_back(data);
+    }
 
+    if (not _impl->handlers.empty()) _impl->thread = new std::thread(&SoapySSDPEndpoint::handlerLoop, this);
+}
+
+SoapySSDPEndpoint::~SoapySSDPEndpoint(void)
+{
+    _impl->done = true;
+    if (_impl->thread != nullptr)
+    {
+        _impl->thread->join();
+        delete _impl->thread;
+    }
+    for (auto &data : _impl->handlers)
+    {
+        delete data;
+    }
+    delete _impl;
+}
+
+void SoapySSDPEndpoint::registerService(const std::string &uuid, const std::string &service, const int ipVer)
+{
+    std::lock_guard<std::mutex> lock(_impl->mutex);
+    this->serviceIpVer = ipVer;
+    this->uuid = uuid;
+    this->service = service;
+    this->periodicNotifyEnabled = true;
+    for (auto &data : _impl->handlers) this->sendNotifyHeader(data, NTS_ALIVE);
+}
+
+std::map<std::string, std::map<int, std::string>> SoapySSDPEndpoint::getServerURLs(const int ipVer, const long timeoutUs)
+{
+    std::unique_lock<std::mutex> lock(_impl->mutex);
+
+    //only needed if this is the first invocation...
+    if (not periodicSearchEnabled)
+    {
+        this->periodicSearchEnabled = true;
+        for (auto &data : _impl->handlers) this->sendSearchHeader(data);
+
+        //wait maximum timeout for replies
+        lock.unlock();
+        std::this_thread::sleep_for(std::chrono::microseconds(timeoutUs));
+        lock.lock();
+    }
+
+    std::map<std::string, std::map<int, std::string>> serverUrls;
+    for (const auto &ipPair : _impl->usnToURL)
+    {
+        if ((ipPair.first & ipVer) == 0) continue;
+        for (auto &pair : ipPair.second)
+        {
+            const auto uuid = uuidFromUSN(pair.first);
+            serverUrls[uuid][ipPair.first] = pair.second.first;
+        }
+    }
+
+    return serverUrls;
+}
+
+void SoapySSDPEndpoint::handlerLoop(void)
+{
     std::string recvAddr;
     char recvBuff[SOAPY_REMOTE_DEFAULT_ENDPOINT_MTU];
 
-    while (not done)
+    //vector of sockets for select operation
+    std::vector<SoapyRPCSocket *> socks;
+    for (auto &data : _impl->handlers) socks.push_back(&data->sock);
+
+    while (not _impl->done)
     {
-        //receive SSDP traffic
-        if (sock.selectRecv(SOAPY_REMOTE_SOCKET_TIMEOUT_US))
+        const int socksReady = SoapyRPCSocket::selectRecvMultiple(socks, SOAPY_REMOTE_SOCKET_TIMEOUT_US);
+        if (socksReady < 0)
         {
-            std::lock_guard<std::mutex> lock(mutex);
+            SoapySDR::logf(SOAPY_SDR_ERROR, "SoapySSDPEndpoint::selectRecvMultiple() = %d", socksReady);
+            return;
+        }
+
+        //locked for all non-blocking routines below
+        std::lock_guard<std::mutex> lock(_impl->mutex);
+
+        for (size_t i = 0; i < _impl->handlers.size(); i++)
+        {
+            if ((socksReady & (1 << i)) == 0) continue;
+            auto data = _impl->handlers[i];
+            auto &sock = data->sock;
+
+            //receive SSDP traffic
             int ret = sock.recvfrom(recvBuff, sizeof(recvBuff), recvAddr);
             if (ret < 0)
             {
@@ -218,38 +285,44 @@ void SoapySSDPEndpoint::handlerLoop(SoapySSDPEndpointData *data)
             if (header.getLine0() == "NOTIFY * HTTP/1.1") this->handleNotifyRequest(data, header, recvAddr);
         }
 
-        //locked for all non-blocking routines below
-        std::lock_guard<std::mutex> lock(mutex);
         const auto timeNow = std::chrono::high_resolution_clock::now();
         const auto triggerExpired = timeNow + std::chrono::seconds(TRIGGER_TIMEOUT_SECONDS);
 
         //remove old cache entries
-        auto it = data->usnToURL.begin();
-        while (it != data->usnToURL.end())
+        for (auto &ipPair : _impl->usnToURL)
         {
-            auto &expires = it->second.second;
-            if (expires > timeNow) ++it;
-            else data->usnToURL.erase(it++);
+            auto &usnToURL = ipPair.second;
+            auto it = usnToURL.begin();
+            while (it != usnToURL.end())
+            {
+                auto &expires = it->second.second;
+                if (expires > timeNow) ++it;
+                else usnToURL.erase(it++);
+            }
         }
 
-        //check trigger for periodic search
-        if (periodicSearchEnabled and data->lastTimeSearch > triggerExpired)
+        //handle periodic messages
+        for (auto &data : _impl->handlers)
         {
-            this->sendSearchHeader(data);
-        }
+            //check trigger for periodic search
+            if (this->periodicSearchEnabled and data->lastTimeSearch > triggerExpired)
+            {
+                this->sendSearchHeader(data);
+            }
 
-        //check trigger for periodic notify
-        if (periodicNotifyEnabled and data->lastTimeNotify > triggerExpired)
-        {
-            this->sendNotifyHeader(data, NTS_ALIVE);
+            //check trigger for periodic notify
+            if (this->periodicNotifyEnabled and data->lastTimeNotify > triggerExpired)
+            {
+                this->sendNotifyHeader(data, NTS_ALIVE);
+            }
         }
     }
 
     //disconnect notification when done
-    if (done)
+    if (_impl->done)
     {
-        std::lock_guard<std::mutex> lock(mutex);
-        this->sendNotifyHeader(data, NTS_BYEBYE);
+        std::lock_guard<std::mutex> lock(_impl->mutex);
+        for (auto &data : _impl->handlers) this->sendNotifyHeader(data, NTS_BYEBYE);
     }
 }
 
@@ -280,7 +353,7 @@ void SoapySSDPEndpoint::sendSearchHeader(SoapySSDPEndpointData *data)
 
 void SoapySSDPEndpoint::sendNotifyHeader(SoapySSDPEndpointData *data, const std::string &nts)
 {
-    if (not serviceRegistered) return; //do we have a service to advertise?
+    if ((serviceIpVer & data->ipVer) == 0) return; //do we have a service to advertise?
 
     auto hostURL = SoapyURL(data->groupURL);
     hostURL.setScheme(""); //no scheme name
@@ -303,7 +376,7 @@ void SoapySSDPEndpoint::sendNotifyHeader(SoapySSDPEndpointData *data, const std:
 
 void SoapySSDPEndpoint::handleSearchRequest(SoapySSDPEndpointData *data, const SoapyHTTPHeader &request, const std::string &recvAddr)
 {
-    if (not serviceRegistered) return; //do we have a service to advertise?
+    if ((serviceIpVer & data->ipVer) == 0) return; //do we have a service to advertise?
 
     if (request.getField("MAN") != "\"ssdp:discover\"") return;
     const auto st = request.getField("ST");
@@ -368,7 +441,9 @@ void SoapySSDPEndpoint::handleRegisterService(SoapySSDPEndpointData *data, const
     //handle byebye from notification packets
     if (header.getField("NTS") == NTS_BYEBYE)
     {
-        data->usnToURL.erase(usn);
+        auto &usnToURL = _impl->usnToURL[data->ipVer];
+        SoapySDR::logf(SOAPY_SDR_DEBUG, "SoapySSDP removed %s [%s] %s IPv%d", usnToURL[usn].first.c_str(), uuidFromUSN(usn).c_str(), data->ethName.c_str(), data->ipVer);
+        usnToURL.erase(usn);
         return;
     }
 
@@ -376,9 +451,9 @@ void SoapySSDPEndpoint::handleRegisterService(SoapySSDPEndpointData *data, const
     const auto location = header.getField("LOCATION");
     if (location.empty()) return;
     const SoapyURL serverURL("tcp", SoapyURL(recvAddr).getNode(), SoapyURL(location).getService());
-    SoapySDR::logf(SOAPY_SDR_DEBUG, "SoapyRemote discovered %s", serverURL.toString().c_str());
+    SoapySDR::logf(SOAPY_SDR_DEBUG, "SoapySSDP discovered %s [%s] %s IPv%d", serverURL.toString().c_str(), uuidFromUSN(usn).c_str(), data->ethName.c_str(), data->ipVer);
 
     //register the server
     const auto expires = std::chrono::high_resolution_clock::now() + std::chrono::seconds(getCacheDuration(header));
-    data->usnToURL[usn] = std::make_pair(serverURL.toString(), expires);
+    _impl->usnToURL[data->ipVer][usn] = std::make_pair(serverURL.toString(), expires);
 }
