@@ -1,10 +1,11 @@
-// Copyright (c) 2015-2017 Josh Blum
+// Copyright (c) 2015-2018 Josh Blum
 // Copyright (c) 2016-2016 Bastille Networks
 // SPDX-License-Identifier: BSL-1.0
 
 #include "ClientHandler.hpp"
 #include "ServerStreamData.hpp"
 #include "LogForwarding.hpp"
+#include "SoapyInfoUtils.hpp"
 #include "SoapyRemoteDefs.hpp"
 #include "SoapyURLUtils.hpp"
 #include "SoapyRPCSocket.hpp"
@@ -185,7 +186,10 @@ bool SoapyClientHandler::handleOnce(SoapyRPCUnpacker &unpacker, SoapyRPCPacker &
     case SOAPY_REMOTE_GET_HARDWARE_INFO:
     ////////////////////////////////////////////////////////////////////
     {
-        packer & _dev->getHardwareInfo();
+        auto info = _dev->getHardwareInfo();
+        //insert the server version using the remote: key prefix
+        info["remote:version"] = SoapyInfo::getServerVersion();
+        packer & info;
     } break;
 
     ////////////////////////////////////////////////////////////////////
@@ -468,6 +472,32 @@ bool SoapyClientHandler::handleOnce(SoapyRPCUnpacker &unpacker, SoapyRPCPacker &
         packer & _dev->deactivateStream(data.stream, flags, timeNs);
     } break;
 
+    ////////////////////////////////////////////////////////////////////
+    case SOAPY_REMOTE_SETUP_STREAM_BYPASS:
+    ////////////////////////////////////////////////////////////////////
+    {
+        char direction = 0;
+        std::string format;
+        std::vector<size_t> channels;
+        SoapySDR::Kwargs args;
+        unpacker & direction;
+        unpacker & format;
+        unpacker & channels;
+        unpacker & args;
+
+        //create stream
+        auto stream = _dev->setupStream(direction, format, channels, args);
+
+        //load data structure
+        auto &data = _streamData[_nextStreamId];
+        data.streamId = _nextStreamId++;
+        data.device = _dev;
+        data.stream = stream;
+        data.format = format;
+
+        packer & data.streamId;
+    } break;
+
     ////////////////////////////////////////////////////////////////////
     case SOAPY_REMOTE_LIST_ANTENNAS:
     ////////////////////////////////////////////////////////////////////
