@@ -1,4 +1,4 @@
-// Copyright (c) 2015-2017 Josh Blum
+// Copyright (c) 2015-2018 Josh Blum
 // Copyright (c) 2016-2016 Bastille Networks
 // SPDX-License-Identifier: BSL-1.0
 
@@ -67,12 +67,16 @@ void ServerStreamData::startStatThread(void)
 void ServerStreamData::stopThreads(void)
 {
     done = true;
-    assert(streamThread != nullptr);
-    assert(statusThread != nullptr);
-    streamThread->join();
-    statusThread->join();
-    delete streamThread;
-    delete statusThread;
+    if (streamThread != nullptr)
+    {
+        streamThread->join();
+        delete streamThread;
+    }
+    if (statusThread != nullptr)
+    {
+        statusThread->join();
+        delete statusThread;
+    }
 }
 
 static void setThreadPrioWithLogging(const double priority)
@@ -123,6 +127,11 @@ void ServerStreamData::recvEndpointWork(void)
                 endpoint->writeStatus(ret, chanMask, flags, timeNs);
                 break; //discard after error, this may have been invalid flags or time
             }
+            if (elemsLeft < (size_t)ret)
+            {
+                SoapySDR_logf(SOAPY_SDR_ERROR, "Server-side receive endpoint: device->writeStream wrote more elements than requested");
+                break; //stop after error
+            }
             elemsLeft -= ret;
             incrementBuffs(buffs, ret, elemSize);
             if (elemsLeft == 0) break;
